<?php

if (!function_exists('openssl_encrypt'))
{
	trigger_error('OpenSSL functions are not available', E_USER_ERROR);
	return 0;
}

class ADODB_Encrypt_OpenSSL
{

	private $method                       = null;
	private $tag_length                   = 16;

	function __construct($method='AES-256-GCM')
	{
		$this->method = $method;
	}

	function write($data, $key)
	{
		$ivSize = openssl_cipher_iv_length($this->method);
		$iv = openssl_random_pseudo_bytes($ivSize, $crypto_strong);

		$encrypted = openssl_encrypt($data, $this->method, $key, OPENSSL_RAW_DATA, $iv, $tag, null, $this->tag_length);
		$encrypted = ($tag . $iv . $encrypted);
		return $encrypted;
	}

	function read($data, $key)
	{
		$ivSize = openssl_cipher_iv_length($this->method);
		$tag = substr($data, 0, $this->tag_length);
		$iv = substr($data, $this->tag_length, $ivSize);
		$data = openssl_decrypt(substr($data, $ivSize + $this->tag_length), $this->method, $key, OPENSSL_RAW_DATA, $iv, $tag);
		return $data;
	}
}
return 1;

?>
